BEGIN {
  unless(eval q{ use 5.006; 1}) {
    print "Perl 5.006 or better required\n";
    exit;
  }
}

# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v5.037.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Module::Build" => "0.28",
    "perl" => "5.006"
  },
  "dist_abstract" => "Wrap git command-line interface without blocking",
  "dist_author" => [
    "Graham Ollis <plicease\@cpan.org>"
  ],
  "dist_name" => "AnyEvent-Git-Wrapper",
  "dist_version" => "0.08",
  "license" => "perl",
  "module_name" => "AnyEvent::Git::Wrapper",
  "recommends" => {
    "EV" => 0
  },
  "recursive_test_files" => 1,
  "requires" => {
    "AnyEvent" => 0,
    "AnyEvent::Open3::Simple" => "0.83",
    "File::pushd" => 0,
    "Git::Wrapper" => "0.030",
    "Scalar::Util" => 0,
    "perl" => "5.006"
  },
  "script_files" => [],
  "test_requires" => {
    "Sort::Versions" => 0,
    "Test::Deep" => 0,
    "Test::Exception" => 0,
    "perl" => "5.006"
  }
);


my %fallback_build_requires = (
  "Module::Build" => "0.28",
  "Sort::Versions" => 0,
  "Test::Deep" => 0,
  "Test::Exception" => 0,
  "perl" => "5.006"
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);

$build->create_build_script;
